@@ -1,4 +1,5 @@
 """Allow users to access circuitry symbols directly."""
 from circuitry.circuitry import\
     bit, constant, input, input_one, input_two, output,\
-    bits, constants, inputs, outputs
+    bits, constants, inputs, outputs,\
+    bits_type, synthesize
